//////////////////////////////////////////////////////////////////////
// RUDI v0.1 - an open source Christmas game
// Copyright (C) 2007 Christian Behrenberg
//--------------------------------------------------------------------
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation;
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this program; if not, write to the Free Software Foundation, Inc.,
// 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//--------------------------------------------------------------------
// module       : actors
// created (by) : Christian Behrenberg, 2007-11-17
//--------------------------------------------------------------------

//--------------------------------------------------------------------
// ASSIGNS A PATH TO AN ACTOR DETERMINED BY THE PATHS _WED_ NAME
//
// params: ent      - actor entity
//         pathName - WED pathname
//--------------------------------------------------------------------
void act_path_assign (ENTITY* ent, char* pathName)
{
	path_set(ent, pathName);
}

//--------------------------------------------------------------------
// MAKES AN ACTOR MOVE ON A SPLINE PATH
//
// ENT is the actor, SPEED determines the speed for _one_ frame and
// OFFSET determines the displaced start on the path.
//
// Note: skill90 - skill93 are used by this function for internal
//       purposes!
//--------------------------------------------------------------------
void act_path_run (ENTITY* ent, var speed, var offset)
{
	//skill90 stores the progress on the path
	//skill91-93 the last position (xyz)

	// Move on the path

		path_spline(ent, ent->x, offset + ent->skill90);
		ent->skill90 += speed * time_step;

	// Rotate so that the entity looks along the path

		vec_diff(vecTemp, ent->x, my.skill91);
		vec_to_angle(vecTemp.x, vecTemp);
		ang_lerp(my.pan, my.pan, vecTemp.x, 0.2 * time_step);
		vec_set(my.skill91, ent->x);
}


//--------------------------------------------------------------------
// ACTOR PATROLING ALONG A PATH
//
// Skill1 = idle anim speed
// Skill2 = walk anim speed
// Skill3 = Rudi dist min stand
// Skill4 = run time min
// Skill5 = walk speed
// Skill6 = seed
//--------------------------------------------------------------------
void act_patrolPath ()
{
	// Initialization

		// Wait to enable skills
			wait(3);

		// Assign path
			act_path_assign(my, my.string2);

		// Collisionhull
			c_setminmax(my);
			vec_scale(my.min_x, 0.75);
			vec_scale(my.max_x, 0.75);

		// Internals

		   var seed;
		   var stopAndGo = 1;
	
				if (my.skill6 == 0) {
					seed = random(30000);
				} else {
					seed = my.skill6;
					stopAndGo = 0;
				}

		   var speedSeed = my.skill5;

			var flag = 0;
			var timeout = 0;
			int state = 1;

	// Main loop
	while (1) {

		flag = 0; // Reset stop condition

		// Player approaches actor
		if (Rudi) {
			if (vec_dist(my.x, Rudi->x) < my.skill3) {
				flag += 1;

				// If we are part of a caravane, register frame
				if (!stopAndGo) {act_patrolPathCaravaneStop = total_frames;}
			}

		}

		// Default patrol state check
		if (stopAndGo) {

			// State time is over
			if (timeout < 0) {
				state = (++state % 2); // 1->0 or 0->1

				// Randomize state length
				timeout = 16 * ((random(my.skill4) + 3) +
							((1-state) * random(my.skill4)));

			} else { // Decrease timer
				timeout -= time_step;
			}

			// If we are in waiting state, enable flag
			flag += state;

		} else {

			//We are part of a caravane and only stop if all stop
			flag += (abs(act_patrolPathCaravaneStop - total_frames) < 3);

		}

		// Walking
		if (flag == 0) {

			// Run on path
				act_path_run(my, speedSeed, seed);

			// Trace down to stand on my feet
		 		c_trace(my.x, vector(my.x, my.y, my.z - 1000), IGNORE_ME | IGNORE_PASSABLE | IGNORE_SPRITES | IGNORE_PUSH);
				my.z = target.z - my.min_z;

			// Animation
				ent_animate(my, "walk", my.skill2 * (total_ticks + seed), ANM_CYCLE);

		} else { // Standing

			// Animation
				ent_animate(my, "idle", my.skill1 * (total_ticks + seed), ANM_CYCLE);
		}

		wait(1);
	}
}

//--------------------------------------------------------------------
// ACTOR WATCHES RUDI
//
// Skill1 determines the IDLE animation speed
// Skill2 determines the WALK animation speed
// Skill3 determines the viewing RANGE
//--------------------------------------------------------------------
void act_watchRudi ()
{
	// Use my real size
		c_setminmax(my);
		vec_scale(my.min_x, 0.6);	// Shrink
		vec_scale(my.max_x, 0.6);

	// Animation offset
		var seed = random(360);

	while (1) {

		if (Rudi) {

			// Player approaches the actor
			if (vec_dist(my.x, Rudi.x) < my.skill3) {

				// Calculate direct rotation
					vec_diff(vecTemp.x, Rudi->x, my.x);
					vec_to_angle(vecTemp.x, vecTemp.x);

				// Interpolate rotation and save in skill10 the
				// amount of the rotation (angle difference)

					my.skill10 = ang(my.pan);
					ang_lerp(my.pan, my.pan, vecTemp.x, 0.3 * time_step);
					my.skill10 -= ang(my.pan);

					my.roll = 0;
					my.tilt = 0;

				// Depending on the amount of the rotated degrees
				// the actor stands still (idle) or plays the walk
				// animation to prevent "hovered" rotation

					// Above a certain rotation we play the walk animation
					if (abs(ang(my.skill10)) > 1) {
						ent_animate(my, "walk", ((total_ticks + seed) * my.skill2)%100, ANM_CYCLE);
					} else { // Stand still
						ent_animate(my, "idle", ((total_ticks + seed) * my.skill1)%100, ANM_CYCLE);
					}

			} else {
				// Player is far away, so just play the idle animation
				ent_animate(my, "idle", ((total_ticks + seed) * my.skill1)%100, ANM_CYCLE);
			}
		}

		wait(1);
	}
}


//--------------------------------------------------------------------
// ACTOR PLAYING AN ANIMATION IN LOOP
//
// params: animName - animation frame name
//--------------------------------------------------------------------
void act_loop_anim (STRING* animName)
{
	// Use my real size for collision

		c_setminmax(my);

		vec_scale(my.min_x, 0.6);	// Shrink
		vec_scale(my.max_x, 0.6);

	// Animation offset

		var seed = random(360);

	while (1) {

		ent_animate(my, animName, ((total_ticks + seed) * my.skill1)%100, ANM_CYCLE);

		wait(1);
	}
}